//
//*****************************************************************************
//*                                                                           *
//* COPYRIGHT (c) Maelzel, Inc.,                                              *
//*                                                                           *
//*          Redistribution and use of this code and the algorithms herein    *
//*          contained, with or without modification, is permitted provided   *
//*          that the following conditions are met:                           *
//*          * Redistribution of this code and the algorithms herein          *
//*            contained must retain the above copyright notice, this list    *
//*            of conditions, and the following disclaimer.                   *
//*          * Any use of this code and the algorithms herein contained as    *
//*            text or in binary form must attribute, in a reasonable         *
//*            manner, the provenance and authorship of this code as          *
//*            shown in the copyright above.                                  *
//*                                                                           *
//* DISCLAIMER This code and the algorithms herein contained are provided     *
//*            by the developer ''as is'' without any expressed or implied    *
//*            warranties, including, but not limited to, fitness for any     *
//*            particular purpose or any consequential damage that may        *
//*            occur as a consequence of use or association with this code    *
//*            or the algorithms herein contained.                            *
//*                                                                           *
//* TOP      Midwife.java                                                     *
//* CLASS    Console.java (Console)                                           *
//* AUTHOR   Ovi Chris Rouly                                                  *
//*                                                                           *
//* DATE     04/05/2014                                                       *
//* REVISION X.0.2                                                            *
//* BYTES    22,694                                                           *
//* TIME     21:44                                                            *
//* PURPOSE  This GUI informs the user about the current working status of    *
//*          the Midwife program and its processes. It also permits the user  *
//*          some control of the simulation.                                  *
//* METHOD   Uses network connectivity and performance metrics observation.   *
//* DETAILS  Programmer notes embedded in source below & CISIS, Rouly, 2014.  *
//* ENVIRON  Developed on a 64-bit Intel Core2Quad, 2.33GHz, with 4GB RAM.    *
//*          The Zotac m-ITX motherboard had board-based Nvidia 9300 graphics *
//*          acceleration. The OS was MS Windows 8 Pro.  Java IDE provided by *
//*          JGrasp version 2.0.0_08, The Java compiler was version 1.7.0_17. *
//*          Verification tests were run in Java JRE build 1.7.0_17-b02 with  *
//*          client VM build 23.7-b01.                                        *
//*                                                                           *
//*****************************************************************************
//

package gui;                                                 // home directory.

import java.awt.Color;                                        // Java language.
import java.awt.Dimension;                                    //         ditto.
import java.awt.event.ActionEvent;                            //         ditto.
import java.awt.event.ActionListener;                         //         ditto.
import java.awt.event.WindowAdapter;                          //         ditto.
import java.awt.event.WindowEvent;                            //         ditto.
import java.awt.Font;                                         //         ditto.
import java.awt.Frame;                                        //         ditto.
import java.util.ArrayList;                                   //         ditto.
import javax.swing.border.BevelBorder;                        //         ditto.
import javax.swing.BorderFactory;                             //         ditto.
import javax.swing.DefaultListSelectionModel;                 //         ditto.
import javax.swing.event.ListSelectionEvent;                  //         ditto.
import javax.swing.event.ListSelectionListener;               //         ditto.
import javax.swing.GroupLayout;                               //         ditto.
import javax.swing.JButton;                                   //         ditto.
import javax.swing.JFrame;                                    //         ditto.
import javax.swing.JLabel;                                    //         ditto.
import javax.swing.JList;                                     //         ditto.
import javax.swing.JOptionPane;                               //         ditto.
import javax.swing.JScrollPane;                               //         ditto.
import javax.swing.LayoutStyle;                               //         ditto.
import javax.swing.ListSelectionModel;                        //         ditto.
import javax.swing.SwingConstants;                            //         ditto.

// ----------------------------------------------------------------------------
// Console

public class Console extends Frame
{

  // Constants.
  final static long serialVersionUID = 0L;  // required for serialization only.
  final static int CONSOLE_WIDTH = 350;    // a non-VGA video mode screen size.
  final static int CONSOLE_HEIGHT = 275;   // a non-VGA video mode screen size.
  final static Dimension F_DIM = new Dimension(CONSOLE_WIDTH, CONSOLE_HEIGHT );
  final static int BUTTON_WIDTH = 107;            // an arbitrary button width.
  final static int BUTTON_HEIGHT = 20;           // an arbitrary button height.
  final static Dimension B_DIM = new Dimension ( BUTTON_WIDTH, BUTTON_HEIGHT );

  // Variables.
  private String [] ipaSubnetArray = null;   // list of cluster connected IPAs.
  private static Boolean freezeXpansion = true;   // controls Midwife activity.

  // Objects.
  private JScrollPane scrollPane;            // pointer to the background pane.
  private JList < Object > ipaClusterList;       // the actual list in cluster.
  private JButton freezeExpansion;             // freeze expansion of this IPA.
  private JButton resetAllAgents;              // reset all Agents on this IPA.
  private JLabel agentCountLabel;                   // statement of fact label.
  public JLabel agentCountValue;                 // the actual count of agents.
  private JLabel ipaClusterListLabel;               // statement of fact label.
  private JFrame exitOptionPrompt;                   // the exit option prompt.

  // --------------------------------------------------------------------------
  /** this constructor instantiates an instance of the Console GUI object. */

  public Console ( ArrayList < String > ipaSubnet )
  {

    ipaSubnetArray = convertAListToArray ( ipaSubnet );        // prepare data.
    initComponents ( ipaSubnetArray );                 // initializes the form.
    ipaSubnetArray = null;                  // return the memory to the system.

    setLocationRelativeTo ( null );                // center the form on start.
    setVisible ( true );                            // make everything visible.
    repaint ();                    // update the screen with all buffered data.

    freezeXpansion = true;              // initially the Midwives are inactive.

    System.gc ();        // clear away any garbage made during the above steps.

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this returns a string array converted from a string array list. */

  public String [] convertAListToArray ( ArrayList < String > aList )
  {

    String [] array = new String [ aList.size () ];

    Object [] object = aList.toArray ();     // from ArrayList to Object array.
    for ( int i = 0; i < aList.size (); i++ )            // objects to strings.
    {

      array [ i ] = object [ i ].toString ();          // convert the elements.

    }
    object = null;                          // return the memory to the system.

    return array;

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this method causes the form to close and the system to exit. */

  private void exitForm ( WindowEvent event )
  {

    int response = JOptionPane.showConfirmDialog (
      exitOptionPrompt,
      "Terminate Midwife?",
      "Confirm Choice",
      JOptionPane.YES_NO_OPTION );

    switch ( response )
    {

      case JOptionPane.YES_OPTION:
        System.exit ( 0 );
        break;                                          // choice made to quit.

      case JOptionPane.NO_OPTION:
        break;                                      // choice made to continue.

      case JOptionPane.CLOSED_OPTION:
        break;                           // closed without control choice made.

    }

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this inner class respond to clicks on the freeze expansion button. */

  private class FreezeExpansion implements ActionListener
  {

    @Override
    public void actionPerformed ( ActionEvent e )
    {

      // at boot-time, the freezeXpansion mode is TRUE, Midwife is inactive.
      // The first pass, and any other pass, thru here is on user key click.
      freezeXpansion = !freezeXpansion;         // toggle the operational mode.
      if ( freezeXpansion )
      {

        freezeExpansion.setText ( "Permit Expansion" );         // button text.

      }
      else
      {

        agentCountValue.setText ( "wait  " );                 // field display.
        freezeExpansion.setText ( "Freeze Expansion" );         // button text.

      }

    }
  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this gets the value of the user choice to continue expanding Agent
      load. */

  public boolean getFreezeXpansion ()
  {

    return freezeXpansion;

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // WARNING: long method.
  /** this method is called from within the constructor to initialize the
      form. */

  private void initComponents ( String [] ipaStrings )
  {

    scrollPane = new JScrollPane ();       // pointer to the list scroll panel.
    ipaClusterList = new JList < Object > ( ipaStrings );  // cluster IPA list.
    freezeExpansion = new JButton ();          // freeze expansion of this IPA.
    resetAllAgents = new JButton ();           // reset all Agents on this IPA.
    agentCountLabel = new JLabel ();                // statement of fact label.
    agentCountValue = new JLabel ();             // the actual count of agents.
    ipaClusterListLabel = new JLabel ();            // statement of fact label.

    ///////////////////////////////////////////////////////////////////////////

    setBackground ( new Color ( 0, 153, 153 ) );
    setTitle ( "Midwife: Cluster Load Distribution" );
    setName ( "Midwife" );
    setFont ( new Font ( "Calibri", Font.PLAIN, 10 ) );

    setMaximumSize ( F_DIM );                  // a VGA video mode screen size.
    setMinimumSize ( F_DIM );                  // a VGA video mode screen size.
    setPreferredSize ( F_DIM );                // a VGA video mode screen size.
    setResizable ( false );            // disables the resizable frame feature.
    setSize ( F_DIM );                        // a defined (above) screen size.

    ///////////////////////////////////////////////////////////////////////////

    addWindowListener ( new WindowAdapter ()
    {

      @Override
      public void windowClosing ( WindowEvent event )
      {

        exitForm ( event );

      }

    } );

    ///////////////////////////////////////////////////////////////////////////

    ipaClusterList.setBorder ( BorderFactory.createEtchedBorder () );
    ipaClusterList.setFont ( new Font ( "Microsoft Sans Serif", 0, 14 ) );
    scrollPane.setViewportView ( ipaClusterList );
    ipaClusterList.setEnabled ( false );

    ///////////////////////////////////////////////////////////////////////////

    ipaClusterListLabel.setHorizontalAlignment ( SwingConstants.CENTER );
    ipaClusterListLabel.setFont ( new Font ( "Microsoft Sans Serif", 1, 12 ) );
    ipaClusterListLabel.setText ( "IPAs found in Cluster" );
    ipaClusterListLabel.setToolTipText ( "IPAs in cluster at startup." );
    ipaClusterListLabel.setHorizontalTextPosition ( SwingConstants.CENTER );
    ipaClusterListLabel.getAccessibleContext ().
      setAccessibleName ( "clusterIPAs" );

    ///////////////////////////////////////////////////////////////////////////

    agentCountLabel.setBackground ( new Color ( 0, 153, 153 ) );
    agentCountLabel.setHorizontalAlignment ( SwingConstants.RIGHT );
    agentCountLabel.setHorizontalTextPosition ( SwingConstants.CENTER );
    agentCountLabel.setFont ( new Font ( "Microsoft Sans Serif", 1, 14 ) );
    agentCountLabel.setText ( "Active agents:" );
    agentCountLabel.setToolTipText ( "The count of agents now active." );
    agentCountLabel.setMaximumSize ( B_DIM );
    agentCountLabel.setMinimumSize ( B_DIM );
    agentCountLabel.setPreferredSize ( B_DIM );
    agentCountLabel.setRequestFocusEnabled ( false );
    agentCountLabel.getAccessibleContext ().
      setAccessibleName ( "activeAgentLabel" );

    ///////////////////////////////////////////////////////////////////////////

    agentCountValue.setBackground ( new Color ( 0, 153, 153 ) );
    agentCountValue.setHorizontalAlignment ( SwingConstants.LEFT );
    agentCountValue.setHorizontalTextPosition ( SwingConstants.CENTER );
    agentCountValue.setFont ( new Font ( "Microsoft Sans Serif", 1, 14 ) );
    agentCountValue.setText ( "Frozen" );
    agentCountValue.setToolTipText ( "The Midwife operational loading mode." );
    agentCountValue.setMaximumSize ( B_DIM );
    agentCountValue.setMinimumSize ( B_DIM );
    agentCountValue.setPreferredSize ( B_DIM );
    agentCountValue.setRequestFocusEnabled ( false );
    agentCountValue.getAccessibleContext ().
      setAccessibleName ( "activeAgentCount" );

    ///////////////////////////////////////////////////////////////////////////

    freezeExpansion.addActionListener ( new FreezeExpansion () );
    freezeExpansion.setText ( "Permit Expansion" );
    freezeExpansion.setBorder ( BorderFactory.createBevelBorder (
      BevelBorder.RAISED ) );
    freezeExpansion.setHideActionText ( true );
    freezeExpansion.setEnabled ( true );

    ///////////////////////////////////////////////////////////////////////////

    resetAllAgents.addActionListener ( new Terminate () );
    resetAllAgents.setText ( "Terminate" );
    resetAllAgents.setBorder ( BorderFactory.createBevelBorder (
      BevelBorder.RAISED ) );
    resetAllAgents.setHideActionText ( true );
    resetAllAgents.setEnabled ( true );

    ///////////////////////////////////////////////////////////////////////////

    GroupLayout layout = new GroupLayout ( this );
    this.setLayout ( layout );

    layout.setHorizontalGroup (
      layout.createParallelGroup ( GroupLayout.Alignment.LEADING )
      .addGroup ( layout.createSequentialGroup ()
      .addGap ( 48, 48, 48 )
      .addGroup ( layout.createParallelGroup ( GroupLayout.Alignment.TRAILING )
      .addGroup ( layout.createSequentialGroup ()
      .addComponent ( agentCountLabel, GroupLayout.PREFERRED_SIZE, 142,
                      GroupLayout.PREFERRED_SIZE )
      .addGap ( 10, 10, 10 )
      .addComponent ( agentCountValue, GroupLayout.PREFERRED_SIZE, 98,
                      GroupLayout.PREFERRED_SIZE ) )
      .addComponent ( ipaClusterListLabel, GroupLayout.PREFERRED_SIZE, 250,
                      GroupLayout.PREFERRED_SIZE )
      .addGroup ( layout.createSequentialGroup ()
      .addComponent ( freezeExpansion, GroupLayout.PREFERRED_SIZE, 120,
                      GroupLayout.PREFERRED_SIZE )
      .addPreferredGap ( LayoutStyle.ComponentPlacement.UNRELATED )
      .addComponent ( resetAllAgents, GroupLayout.PREFERRED_SIZE, 120,
                      GroupLayout.PREFERRED_SIZE ) ) )
      .addContainerGap ( 52, Short.MAX_VALUE ) )
      .addGroup ( GroupLayout.Alignment.TRAILING, layout.
      createSequentialGroup ()
      .addContainerGap ( GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE )
      .addComponent ( scrollPane, GroupLayout.PREFERRED_SIZE, 154,
                      GroupLayout.PREFERRED_SIZE )
      .addContainerGap ( GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE ) ) );

    layout.setVerticalGroup (
      layout.createParallelGroup ( GroupLayout.Alignment.LEADING )
      .addGroup ( layout.createSequentialGroup ()
      .addGap ( 30, 30, 30 )
      .addGroup ( layout.createParallelGroup ( GroupLayout.Alignment.BASELINE )
      .addComponent ( agentCountLabel, GroupLayout.PREFERRED_SIZE,
                      GroupLayout.DEFAULT_SIZE,
                      GroupLayout.PREFERRED_SIZE )
      .addComponent ( agentCountValue, GroupLayout.PREFERRED_SIZE,
                      GroupLayout.DEFAULT_SIZE,
                      GroupLayout.PREFERRED_SIZE ) )
      .addPreferredGap ( LayoutStyle.ComponentPlacement.RELATED )
      .addComponent ( ipaClusterListLabel )
      .addPreferredGap ( LayoutStyle.ComponentPlacement.UNRELATED )
      .addComponent ( scrollPane, GroupLayout.PREFERRED_SIZE, 77,
                      GroupLayout.PREFERRED_SIZE )
      .addGap ( 18, 18, 18 )
      .addGroup ( layout.createParallelGroup ( GroupLayout.Alignment.BASELINE )
      .addComponent ( freezeExpansion, GroupLayout.PREFERRED_SIZE, 30,
                      GroupLayout.PREFERRED_SIZE )
      .addComponent ( resetAllAgents, GroupLayout.PREFERRED_SIZE, 30,
                      GroupLayout.PREFERRED_SIZE ) )
      .addContainerGap ( 62, Short.MAX_VALUE ) ) );


    ///////////////////////////////////////////////////////////////////////////

    pack ();

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this sets the value of the agent count display. */

  public void setAgentCountValue ( int newValue )
  {

    // functionally defined in BirthRoom.countTheAgents(), displayable here.
    final int SYSTEM_FROZEN = Integer.MAX_VALUE;

    // choose to change the display only if the newValue is active.
    if ( newValue != SYSTEM_FROZEN )
    {

      agentCountValue.setText ( new Integer ( newValue ).toString () );  // nr.

    }

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this inner class respond to clicks on the reset all agents button. */

  private class Terminate implements ActionListener
  {

    @Override
    public void actionPerformed ( ActionEvent e )
    {

      int response = JOptionPane.showConfirmDialog (
        exitOptionPrompt,
        "Terminate Midwife?",
        "Confirm Choice",
        JOptionPane.YES_NO_OPTION );

      switch ( response )
      {

        case JOptionPane.YES_OPTION:
          System.exit ( 0 );
          break;                                        // choice made to quit.

        case JOptionPane.NO_OPTION:
          break;                                    // choice made to continue.

        case JOptionPane.CLOSED_OPTION:
          break;                         // closed without control choice made.

      }

    }
  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this prepares for and updates the displayed IPA subnet list on the
      form. */

  public void updateIPAList ( ArrayList < String > ipaSubnet )
  {

    ipaSubnetArray = convertAListToArray ( ipaSubnet );        // prepare data.

    ipaClusterList = new JList < Object > ( ipaSubnetArray );// clstr IPA list.
    ipaClusterList.setFont ( new Font ( "Microsoft Sans Serif", 0, 14 ) );
    scrollPane.setViewportView ( ipaClusterList );
    setVisible ( true );                            // make everything visible.
    repaint ();                    // update the screen with all buffered data.

    ipaSubnetArray = null;                  // return the memory to the system.

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this causes a wait in milliseconds while checking for exceptions. */

  public void wait ( int milliseconds )
  {

    try
    {

      Thread.sleep ( milliseconds );

    }
    catch ( Exception ie )
    {

      System.out.println ( "ERROR: Console.wait" );
      System.out.println ( "       " + ie );

    }

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this method does nothing. */

  public void template ()
  {
  } // End method.
  // --------------------------------------------------------------------------

// ----------------------------------------------------------------------------
}
