//
//*****************************************************************************
//*                                                                           *
//* COPYRIGHT (c) Maelzel, Inc.,                                              *
//*                                                                           *
//*          Redistribution and use of this code and the algorithms herein    *
//*          contained, with or without modification, is permitted provided   *
//*          that the following conditions are met:                           *
//*          * Redistribution of this code and the algorithms herein          *
//*            contained must retain the above copyright notice, this list    *
//*            of conditions, and the following disclaimer.                   *
//*          * Any use of this code and the algorithms herein contained as    *
//*            text or in binary form must attribute, in a reasonable         *
//*            manner, the provenance and authorship of this code as          *
//*            shown in the copyright above.                                  *
//*                                                                           *
//* DISCLAIMER This code and the algorithms herein contained are provided     *
//*            by the developer ''as is'' without any expressed or implied    *
//*            warranties, including, but not limited to, fitness for any     *
//*            particular purpose or any consequential damage that may        *
//*            occur as a consequence of use or association with this code    *
//*            or the algorithms herein contained.                            *
//*                                                                           *
//* TOP      Midwife.java                                                     *
//* CLASS    GetLocalIPA.java (Get Local IPA)                                 *
//* AUTHOR   Ovi Chris Rouly                                                  *
//*                                                                           *
//* DATE     04/05/2014                                                       *
//* REVISION X.0.2                                                            *
//* BYTES    6,623                                                            *
//* TIME     21:44                                                            *
//* PURPOSE  This establishes a socket, determines the default IPA associated *
//*          with that socket (the Midwife host CPU network adapter           *
//*          effectively) then stores the particulars for external accessor   *
//*          pickup.                                                          *
//* METHOD   Uses network connectivity and performance metrics observation.   *
//* DETAILS  Programmer notes embedded in source below & CISIS, Rouly, 2014.  *
//* ENVIRON  Developed on a 64-bit Intel Core2Quad, 2.33GHz, with 4GB RAM.    *
//*          The Zotac m-ITX motherboard had board-based Nvidia 9300 graphics *
//*          acceleration. The OS was MS Windows 8 Pro.  Java IDE provided by *
//*          JGrasp version 2.0.0_08, The Java compiler was version 1.7.0_17. *
//*          Verification tests were run in Java JRE build 1.7.0_17-b02 with  *
//*          client VM build 23.7-b01.                                        *
//*                                                                           *
//*****************************************************************************
//

package network;                                             // home directory.

import java.net.InetAddress;                                  // Java language.
import java.net.Socket;                                       //         ditto.
import java.net.UnknownHostException;                         //         ditto.

import server.*;                                              // user packages.

// ----------------------------------------------------------------------------
// GetLocalIPA

public class GetLocalIPA
{

  // Constants.

  // Variables.
  private InetAddress localHostIPA = null;    // local host IPA as InetAddress.
  private String localHostStr = "";                // local host IPA as string.

  // Objects.
  private Server server = new Server (); // provides network interface service.

  // --------------------------------------------------------------------------
  /** this is a default constructor. */

  public GetLocalIPA ()
  {
  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this establishes a socket, determines the IPA, stores the particulars,
  // then stores it all for for external access. */

  public void discoverLocalHost ()
  {

    Socket socket = null;
    socket = server.getSocket ();          // create an adapter-default socket.
    InetAddress ipa = socket.getLocalAddress ();               // symbolic IPA.

    try
    {

      this.localHostIPA = InetAddress.getLocalHost ();   // recover system IPA.
      this.localHostStr = InetAddress.getLocalHost ().toString ();  // now Str.

    }
    catch ( UnknownHostException uhe )
    {

      System.out.println ( "ERROR: getLocalHostIPA.discoverLocalHost" );
      System.out.println ( "       " + uhe );

    }

    server.disconnect ( socket );  // formally disconnect from the socket used.
    socket = null;                // return the memory to the operating system.
    server = null;                // return the memory to the operating system.

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this returns the value of the local host's IPA as a String. */

  public String getLocalHostStr ()
  {

    // FOR EXAMPLE: "MyFavoriteMartian/127.0.0.1"
    String result [] = this.localHostStr.toString ().split ( "/" );

    return result [ 1 ];

  } // End method.
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  /** this method does nothing. */

  public void template ()
  {
  } // End method.
  // --------------------------------------------------------------------------

// ----------------------------------------------------------------------------
}
